local idTweaks = LibStub('AceAddon-3.0'):NewAddon('idTweaks', 'AceConsole-3.0')
local AceConfig = LibStub('AceConfig-3.0')
local AceConfigDialog = LibStub('AceConfigDialog-3.0')
local LibSharedMedia = LibStub('LibSharedMedia-3.0')
_G.idTweaks = idTweaks

function idTweaks:setDefaults()
  self.defaults = {
    profile = {
    }
  }
end

function idTweaks:setDatabase()
  self.db = LibStub('AceDB-3.0'):New('idTweaksDB', self.defaults, 'Default')
end

function idTweaks:setOptions()
  self.options = {
    name = 'idTweaks',
    type = 'group',
    args = {
    }
  }

  self.profileOptions = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)
end

function idTweaks:registerOptions()
  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Profiles', self.profileOptions)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Profiles', 'Profiles', 'idTweaks')
end

function idTweaks:registerSlashCommands()
  _G.SLASH_IDTWEAKS1 = '/idt'
  _G.SLASH_IDTWEAKS2 = '/idtweaks'

  SlashCmdList['IDTWEAKS'] = function(msg)
    InterfaceOptionsFrame_OpenToCategory('idTweaks')
    InterfaceOptionsFrame_OpenToCategory('idTweaks')
  end
end

function idTweaks:registerMedia()
  -- Thank you, ShadowedUnitFrames
  LibSharedMedia:Register(LibSharedMedia.MediaType.FONT, "Myriad Condensed Web", "Interface\\AddOns\\idTweaks\\media\\fonts\\Myriad Condensed Web.ttf")
  LibSharedMedia:Register(LibSharedMedia.MediaType.BORDER, "Square Clean", "Interface\\AddOns\\idTweaks\\media\\textures\\ABFBorder")
  LibSharedMedia:Register(LibSharedMedia.MediaType.BACKGROUND, "Chat Frame", "Interface\\ChatFrame\\ChatFrameBackground")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "BantoBar", "Interface\\AddOns\\idTweaks\\media\\textures\\banto")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Smooth",   "Interface\\AddOns\\idTweaks\\media\\textures\\smooth")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Perl",     "Interface\\AddOns\\idTweaks\\media\\textures\\perl")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Glaze",    "Interface\\AddOns\\idTweaks\\media\\textures\\glaze")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Charcoal", "Interface\\AddOns\\idTweaks\\media\\textures\\Charcoal")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Otravi",   "Interface\\AddOns\\idTweaks\\media\\textures\\otravi")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Striped",  "Interface\\AddOns\\idTweaks\\media\\textures\\striped")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "LiteStep", "Interface\\AddOns\\idTweaks\\media\\textures\\LiteStep")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Aluminium", "Interface\\AddOns\\idTweaks\\media\\textures\\Aluminium")
  LibSharedMedia:Register(LibSharedMedia.MediaType.STATUSBAR, "Minimalist", "Interface\\AddOns\\idTweaks\\media\\textures\\Minimalist")
end

function idTweaks:OnInitialize()
  self:setDefaults()
  self:setDatabase()
  self:setOptions()
  self:registerMedia()
  self:registerOptions()
  self:registerSlashCommands()
end

function idTweaks:OnEnable()
end

function idTweaks:OnDisable()
end

function idTweaks:newModule(name, ...)
  local module = self:NewModule(name, ...)

  function module:OnInitialize()
    local defaults = module.defaults or {
      profile = {
        enabled = true
      }
    }

    module.db = idTweaks.db:RegisterNamespace(name, defaults)

    module:initialize()
  end

  function module:OnEnable()
    local configName = 'idTweaks_' .. name

    local options = module.options or {
      name = name,
      type = 'group',
      args = {
        enabled = {
          order = 1,
          name = 'Enabled',
          desc = 'Enables / Disables the Module',
          type = 'toggle',
          get = function()
            return module.db.profile.enabled
          end,
          set = function(_, value)
            module.db.profile.enabled = value
            module:check()
          end,
        },
      },
    }

    AceConfig:RegisterOptionsTable(configName, options)
    AceConfigDialog:AddToBlizOptions(configName, name, 'idTweaks')

    module:check()
  end

  function module:OnDisable()
    module:disable()
  end

  function module:initialize()
  end

  function module:check()
    if module.db.profile.enabled then
      module:enable()
    else
      module:disable()
    end
  end

  function module:enable()
  end

  function module:disable()
  end

  return module
end

idTweaks.table = {
  reverse = function(t)
    local out = {}
    for k, v in pairs(t) do
      out[v] = k
    end
    return out
  end,

  copy = function(a, b)
    for k, v in pairs(b) do
      a[k] = v
    end
  end,

  merge = function(...)
    local out = {}
    for i, t in ipairs({...}) do
      idTweaks.table.copy(out, t)
    end
    return out
  end,
}
