local moduleName = 'ExperienceBar'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceEvent-3.0')
local LibSharedMedia = LibStub('LibSharedMedia-3.0')

local sharedMediaStatusBars = LibSharedMedia:HashTable('statusbar')
local sharedMediaStatusBarsReversed = idTweaks.table.reverse(LibSharedMedia:HashTable('statusbar'))
local sharedMediaFonts = LibSharedMedia:HashTable('font')
local sharedMediaFontsReversed = idTweaks.table.reverse(LibSharedMedia:HashTable('font'))
local sharedMediaBackgrounds = LibSharedMedia:HashTable('background')
local sharedMediaBackgroundsReversed = idTweaks.table.reverse(LibSharedMedia:HashTable('background'))


local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

module.defaults = {
  profile = {
    enabled = true,

    height = 10,
    fullWidth = false,
    font = 'Myriad Condensed Web',
    fontHeight = 12,
    texture = 'Minimalist',
    color = {
      red = 0.2,
      green = 0.4,
      blue = 0.6,
    },
    restedTexture = 'Minimalist',
    restedColor = {
      red = 0.6,
      green = 0.2,
      blue = 0.4,
    },
  },
}

module.options = {
  name = moduleName,
  type = 'group',
  disabled = function()
    return not idTweaks.modules.ActionBars.db.profile.enabled
  end,
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    fullWidth = {
      order = 2,
      name = 'Full Width',
      desc = 'Minimap takes up the full width of the screen',
      type = 'toggle',
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.fullWidth
      end,
      set = function(_, value)
        module.db.profile.fullWidth = value
        module:update()
        idTweaks.modules.Minimap.shape.position:check()
      end
    },

    height = {
      order = 3,
      name = 'Height',
      desc = 'Height of the Experience Bar',
      type = 'range',
      min = 1,
      max = 50,
      step = 1,
      bigStep = 1,
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.height
      end,
      set = function(_, value)
        module.db.profile.height = value
        module:update()
        idTweaks.modules.Minimap.shape.position:check()
      end
    },

    font = {
      order = 4,
      name = 'Font',
      desc = 'Font for the Experience Bar',
      type = 'select',
      dialogControl = 'LSM30_Font',
      values = LibSharedMedia:HashTable('font'),
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.font
      end,
      set = function(_, value)
        module.db.profile.font = value
        module:update()
      end
    },

    fontHeight = {
      order = 5,
      name = 'Font Height',
      desc = 'Height of the font',
      type = 'range',
      min = 9,
      max = 24,
      step = 1,
      bigStep = 1,
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.fontHeight
      end,
      set = function(_, value)
        module.db.profile.fontHeight = value
        module:update()
      end
    },

    color = {
      order = 6,
      name = 'Color',
      desc = 'Color of the Experience Bar',
      type = 'color',
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        local red = module.db.profile.color.red
        local green = module.db.profile.color.green
        local blue = module.db.profile.color.blue
        return red, green, blue
      end,
      set = function(_, red, green, blue)
        module.db.profile.color.red = red
        module.db.profile.color.green = green
        module.db.profile.color.blue = blue
        module:update()
      end
    },

    texture = {
      order = 7,
      name = 'Bar Texture',
      desc = 'Texture of the Experience Bar',
      type = 'select',
      dialogControl = 'LSM30_Statusbar',
      values = LibSharedMedia:HashTable('statusbar'),
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.texture
      end,
      set = function(_, value)
        module.db.profile.texture = value
        module:update()
      end
    },

    restedColor = {
      order = 8,
      name = 'Rested Color',
      desc = 'Color of the Rested Experience Bar',
      type = 'color',
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        local red = module.db.profile.restedColor.red
        local green = module.db.profile.restedColor.green
        local blue = module.db.profile.restedColor.blue
        return red, green, blue
      end,
      set = function(_, red, green, blue)
        module.db.profile.restedColor.red = red
        module.db.profile.restedColor.green = green
        module.db.profile.restedColor.blue = blue
        module:update()
      end
    },

    restedTexture = {
      order = 9,
      name = 'Rested Bar Texture',
      desc = 'Texture of the Rested Experience Bar',
      type = 'select',
      dialogControl = 'LSM30_Statusbar',
      values = LibSharedMedia:HashTable('statusbar'),
      disabled = function()
        return not idTweaks.modules.ActionBars.db.profile.enabled or not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.restedTexture
      end,
      set = function(_, value)
        module.db.profile.restedTexture = value
        module:update()
      end
    },
  },
}

function module:initialize()
  local barName = 'idTweaksExperienceBar'
  local bar = CreateFrame('StatusBar', barName, UIParent)

  bar.background = bar:CreateTexture(barName .. 'Background', 'BACKGROUND')
  bar.background:SetAllPoints(bar)
  bar.background:SetColorTexture(0, 0, 0, 0.5)

  bar.rested = bar:CreateTexture(barName .. 'Rested', 'ARTWORK')

  bar.text = bar:CreateFontString(barName .. 'Text', 'OVERLAY')
  bar.text:SetPoint(MC, bar, MC)
  bar.text:SetTextColor(1, 1, 1, 1)

  module.bar = bar
end

function module:enable()
  -- Don't do anything if the actionbar is not enabled
  if idTweaks.modules.ActionBars.db.profile.enabled == false then return end

  module.bar:Show()

  module:setSize()
  module:setTexture()
  module:setColor()
  module:setFont()

  module:updateExperience()

  module:RegisterEvent('CHAT_MSG_COMBAT_XP_GAIN')
  module:RegisterEvent('PLAYER_XP_UPDATE')
end

function module:disable()
  module.bar:Hide()

  module:UnregisterEvent('CHAT_MSG_COMBAT_XP_GAIN')
  module:UnregisterEvent('PLAYER_XP_UPDATE')
end

function module:update()
  module:setSize()
  module:setTexture()
  module:setFont()
  module:setColor()
  module:updateExperience()
end

function module:setSize()
  module.bar:SetHeight(module.db.profile.height)

  if idTweaks.modules.ActionBars.db.profile.enabled then
    if module.db.profile.fullWidth then
      module.bar:ClearAllPoints()
      module.bar:SetPoint(BL, UIParent, BL, 0, 0)
      module.bar:SetPoint(BR, UIParent, BR, 0, 0)
    else
      module.bar:ClearAllPoints()
      module.bar:SetPoint(BC, UIParent, BC, 0, 0)
      module.bar:SetWidth(idTweaksActionBar1:GetWidth())
    end
  else
    module.bar:SetWidth(0)
    module.bar.rested:SetWidth(0)
  end
end

function module:setTexture()
  module.bar:SetStatusBarTexture(sharedMediaStatusBars[module.db.profile.texture])
  module.bar.rested:SetTexture(sharedMediaStatusBars[module.db.profile.restedTexture])
end

function module:setFont()
  local font = module.db.profile.font
  local fontHeight = module.db.profile.fontHeight
  local fontValue = sharedMediaFonts[font]

  module.bar.text:SetFont(fontValue, fontHeight)
end

function module:setColor()
  module.bar:SetStatusBarColor(
    module.db.profile.color.red,
    module.db.profile.color.green,
    module.db.profile.color.blue
  )

  module.bar.rested:SetColorTexture(
    module.db.profile.restedColor.red,
    module.db.profile.restedColor.green,
    module.db.profile.restedColor.blue,
    1
  )
end

function module:updateExperience()
  local newXp = UnitXP('player')
  local newXpMax = UnitXPMax('player')
  local newXpLeft = newXpMax - newXp

  local oldXp = module.xp or newXp
  local oldXpMax = module.xpMax or newXpMax

  local gainedXp = newXp - oldXp
  local gainedXpRepetitionsLeft = math.ceil(newXpLeft / gainedXp)

  local output = string.format('%s - %s = %s', newXpMax, newXp, newXpLeft)

  if gainedXp > 0 then
    output = string.format(
      '%s (%s reps)',
      output,
      gainedXpRepetitionsLeft
    )
  end

  module.xp = newXp
  module.xpMax = newXpMax
  module.xpRested = GetXPExhaustion()

  module.bar:SetMinMaxValues(0, module.xpMax)
  module.bar:SetValue(module.xp)

  module.bar.text:SetText(output)

  module.bar.rested:SetPoint(ML, module.bar:GetStatusBarTexture(), MR, 0, 0)
  module.bar.rested:SetHeight(module.bar:GetHeight())
  if module.xpRested then
    module.bar.rested:SetPoint(ML, module.bar:GetStatusBarTexture(), MR, 0, 0)
    module.bar.rested:SetWidth(module.xpRested / module.xpMax * module.bar:GetWidth())
  else
    module.bar.rested:SetWidth(0)
  end
end

function module:CHAT_MSG_COMBAT_XP_GAIN()
  module:updateExperience()
end

function module:PLAYER_XP_UPDATE()
  module:updateExperience()
end
