local moduleName = 'Tooltips'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceHook-3.0')

module.defaults = {
  profile = {
    enabled = true,

    anchor = {
      value = 'ANCHOR_CURSOR_RIGHT',
    },
    offsetX = 10,
    offsetY = 10,
  }
}

module.options = {
  name = moduleName,
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    anchor = {
      order = 2,
      name = 'Anchor',
      desc = 'Where to anchor the tooltip to.',
      type = 'select',
      values = {
        ['ANCHOR_CURSOR'] = 'Cursor Center',
        ['ANCHOR_CURSOR_LEFT'] = 'Cursor Left',
        ['ANCHOR_CURSOR_RIGHT'] = 'Cursor Right',
        ['ANCHOR_TOPLEFT'] = 'Top Left',
        ['ANCHOR_TOPRIGHT'] = 'Top Right',
        ['ANCHOR_BOTTOMLEFT'] = 'Bottom Left',
        ['ANCHOR_BOTTOMRIGHT'] = 'Bottom Right',
      },
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.anchor.value
      end,
      set = function(_, value)
        module.db.profile.anchor.value = value
      end,
    },

    offsetX = {
      order = 3,
      name = 'Offset X',
      desc = 'X axis offset',
      type = 'range',
      min = -50,
      max = 50,
      step = 1,
      bigStep = 1,
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.offsetX
      end,
      set = function(_, value)
        module.db.profile.offsetX = value
      end,
    },

    offsetY = {
      order = 4,
      name = 'Offset Y',
      desc = 'Y axis offset',
      type = 'range',
      min = -50,
      max = 50,
      step = 1,
      bigStep = 1,
      disabled = function()
        return not module.db.profile.enabled
      end,
      get = function()
        return module.db.profile.offsetY
      end,
      set = function(_, value)
        module.db.profile.offsetY = value
      end,
    },
  }
}

function module:enable()
  module:SecureHook('GameTooltip_SetDefaultAnchor')
end

function module:disable()
  module:Unhook('GameTooltip_SetDefaultAnchor')
end

function module:GameTooltip_SetDefaultAnchor(tooltip, selfObject)
  local offsetX = module.db.profile.offsetX
  local offsetY = module.db.profile.offsetY

  tooltip:SetOwner(
    selfObject,
    module.db.profile.anchor.value,
    module.db.profile.offsetX,
    module.db.profile.offsetY
  )
end
